local version = "1.9.8"
local yayFrame = nil

function YayMounts:HandleNewCompanion()
	if yayFrame then
      local ct, cst = currentTab, currentSubTab
      YayMounts:Frames()
      currentSubTab = cst
      YayMounts:Frames(ct)
    end
end
--------------------------------------------------------------
--GUI functions for drawing frames
--------------------------------------------------------------
local function DrawMountTab(container, mountType)
  container:SetLayout("Flow")
    
  local selectallbutton = AceGUI:Create("Button")
  selectallbutton:SetText("Select All")
  selectallbutton:SetCallback("OnClick", function() YayMounts:SelectAllMounts(mountType) end)
  container:AddChild(selectallbutton)
    
  local clearallbutton = AceGUI:Create("Button")
  clearallbutton:SetText("Clear All")
  clearallbutton:SetCallback("OnClick", function() YayMounts:ClearMounts(mountType) end)
  container:AddChild(clearallbutton)
  
  local filterButton = AceGUI:Create("EditBox")
  filterButton:SetText("")
  filterButton:SetLabel("Filter")
  filterButton:DisableButton(true)
  filterButton:SetMaxLetters(25)
  
  container:AddChild(filterButton)
  
  local scrollcontainerframe = AceGUI:Create("SimpleGroup")
  scrollcontainerframe:SetLayout("Fill")
  scrollcontainerframe:SetFullWidth(true)
  scrollcontainerframe:SetFullHeight(true)
  container:AddChild(scrollcontainerframe)  
  
  local scrollframe = AceGUI:Create("ScrollFrame")
  scrollcontainerframe:AddChild(scrollframe)
  
  YayMountsState.tabContent = AceGUI:Create("InlineGroup")
  YayMountsState.tabContent:SetFullWidth(true)
  YayMountsState.tabContent:SetLayout("Flow")
  
  filterButton:SetCallback("OnTextChanged", function(_,_,text)
	YayMounts:FilterMounts(text) 
	scrollframe:SetScroll(0)
	scrollframe:DoLayout()
  end)
  
  YayMounts:BuildList(mountType)
  scrollframe:AddChild(YayMountsState.tabContent)
end

local function SelectGroupMounts(container, event, group)
	container:ReleaseChildren()
	if group == "tab1" then
		DrawMountTab(container, YMMountType.ground)
	elseif group == "tab2" then
		DrawMountTab(container, YMMountType.flying)
	elseif group == "tab4" then
		DrawMountTab(container, YMMountType.special)
	end
	currentSubTab = group
end

local function DrawGroup1(container)--MOUNTS
  container:SetLayout("Fill")

  local tab  = AceGUI:Create("TabGroup")
  tab:SetLayout("Flow")
  tab:SetTabs({
	{text="Ground ("..YayMounts.db.profile.misc.NumMounts["Ground"]..")", value="tab1"}, 
	{text="Flying ("..YayMounts.db.profile.misc.NumMounts["Flying"]..")", value="tab2"}, 
	{text="Special ("..YayMounts.db.profile.misc.NumMounts["Special"]..")", value="tab4"}})
  tab:SetCallback("OnGroupSelected", SelectGroupMounts)
  
  if currentSubTab ~= nil then
    tab:SelectTab(currentSubTab)
  else
    tab:SelectTab("tab1")
  end
  container:AddChild(tab)
end

local function DrawGroup2(container)--MOUNT OPTIONS
	container:SetLayout("Flow")
	
	-- No dismount while flying
	local nDWF = AceGUI:Create("CheckBox")
	nDWF:SetLabel(YayMountsLocale.String.NDWF)
	nDWF:SetValue(YayMounts:GetDbValue("settings", "NDWF"))
	nDWF:SetCallback("OnValueChanged",  function() YayMounts:SetDbValue("settings", "NDWF", nDWF:GetValue()) end)
	nDWF:SetFullWidth(true)
	container:AddChild(nDWF)
	
	-- Use Telaari Talbuk (or horde version) in Nagrand
	local telaari = AceGUI:Create("CheckBox")
	telaari:SetLabel(YayMountsLocale.String.TELAARI)
	telaari:SetValue(YayMounts:GetDbValue("settings", "TELAARI"))
	telaari:SetCallback("OnValueChanged",  function() YayMounts:SetDbValue("settings", "TELAARI", telaari:GetValue()) end)
	telaari:SetFullWidth(true)
	container:AddChild(telaari)
	
	-- enable new mounts upon learning them
	local enableNew = AceGUI:Create("CheckBox")
	enableNew:SetLabel(YayMountsLocale.String.ENABLENEW)
	enableNew:SetValue(YayMounts:GetDbValue("settings", "ENABLENEW"))
	enableNew:SetCallback("OnValueChanged",  function() YayMounts:SetDbValue("settings", "ENABLENEW", enableNew:GetValue()) end)
	enableNew:SetFullWidth(true)
	container:AddChild(enableNew)

	-- Emergency random mount if no mounts useable from selected mounts
	local emptyRandom = AceGUI:Create("CheckBox")
	emptyRandom:SetLabel(YayMountsLocale.String.ER)
	emptyRandom:SetValue(YayMounts:GetDbValue("settings", "ER"))
	emptyRandom:SetCallback("OnValueChanged",  function() YayMounts:SetDbValue("settings", "ER", emptyRandom:GetValue()) end)
	emptyRandom:SetFullWidth(true)
	container:AddChild(emptyRandom)

	local forceFlyingMount = AceGUI:Create("CheckBox")
	forceFlyingMount:SetLabel(YayMountsLocale.String.FFM)
	forceFlyingMount:SetValue(YayMounts:GetDbValue("settings", "FFM"))
	forceFlyingMount:SetCallback("OnValueChanged", function() YayMounts:SetDbValue("settings", "FFM", forceFlyingMount:GetValue()) end)
	forceFlyingMount:SetFullWidth(true)
	container:AddChild(forceFlyingMount)

	local sliderRepairMount = AceGUI:Create("Slider") --repair mount slider in decimal %
	sliderRepairMount:SetLabel("Low Durability %")
	sliderRepairMount:SetValue(YayMounts:GetDbValue("misc", "MinDurability")*100)
	sliderRepairMount:SetCallback("OnMouseUp", function() YayMounts:SetDbValue("misc", "MinDurability", (sliderRepairMount:GetValue()/100)) end)
	sliderRepairMount:SetSliderValues(0, 100, 5)
	sliderRepairMount:SetDisabled( not YayMounts:GetDbValue("settings", "URM"))

	local repairMount = AceGUI:Create("CheckBox")
	repairMount:SetLabel(YayMountsLocale.String.URM)
	repairMount:SetValue(YayMounts:GetDbValue("settings", "URM"))
	repairMount:SetCallback("OnValueChanged", function() YayMounts:SetDbValue("settings", "URM", repairMount:GetValue()) sliderRepairMount:SetDisabled(not repairMount:GetValue()) end)
	repairMount:SetFullWidth(true)
  
	container:AddChild(repairMount)
	container:AddChild(sliderRepairMount)
  
	if GetItemCount(37011, false) > 0 then
		local flyingBroomPriority = AceGUI:Create("CheckBox")
		flyingBroomPriority:SetLabel(YayMountsLocale.String.FBP)
		flyingBroomPriority:SetValue(YayMounts:GetDbValue("settings", "FBP"))
		flyingBroomPriority:SetCallback("OnValueChanged", function() YayMounts:SetDbValue("settings", "FBP", flyingBroomPriority:GetValue()) end)
		flyingBroomPriority:SetFullWidth(true)
		container:AddChild(flyingBroomPriority)
	end
end

local function DrawGroup3(container) --CLASS OPTIONS
  container:SetLayout("Flow")
  
  local infoCS = AceGUI:Create("Label")
  infoCS:SetText(YayMountsLocale.String.INFOCS)
  infoCS:SetRelativeWidth(0.9)
  container:AddChild(infoCS)
  
	local shamanCS = AceGUI:Create("CheckBox")
    shamanCS:SetLabel(YayMountsLocale.String.SHAMCS)
    shamanCS:SetValue(YayMounts:GetDbValue("settings", "SHAMAN"))
    shamanCS:SetCallback("OnValueChanged",  function() YayMounts:SetDbValue("settings", "SHAMAN", shamanCS:GetValue()) end)
    shamanCS:SetFullWidth(true)
    container:AddChild(shamanCS)
  
    local rogueCS = AceGUI:Create("CheckBox")
    rogueCS:SetLabel(YayMountsLocale.String.RS)
    rogueCS:SetValue(YayMounts:GetDbValue("settings", "ROGUE"))
    rogueCS:SetCallback("OnValueChanged",  function() YayMounts:SetDbValue("settings", "ROGUE", rogueCS:GetValue()) end)
    rogueCS:SetFullWidth(true)
    container:AddChild(rogueCS)
  
    local mageCS = AceGUI:Create("CheckBox")
    mageCS:SetLabel(YayMountsLocale.String.MS)
    mageCS:SetValue(YayMounts:GetDbValue("settings", "MAGE"))
    mageCS:SetCallback("OnValueChanged",  function() YayMounts:SetDbValue("settings", "MAGE", mageCS:GetValue()) end)
    mageCS:SetFullWidth(true)
    container:AddChild(mageCS)
  
    local dkCS = AceGUI:Create("CheckBox")
    dkCS:SetLabel(YayMountsLocale.String.DKS)
    dkCS:SetValue(YayMounts:GetDbValue("settings", "DEATHKNIGHT"))
    dkCS:SetCallback("OnValueChanged",  function() YayMounts:SetDbValue("settings", "DEATHKNIGHT", dkCS:GetValue()) end)
    dkCS:SetFullWidth(true)
    container:AddChild(dkCS)
  
    local palCS = AceGUI:Create("CheckBox")
    palCS:SetLabel(YayMountsLocale.String.PS)
    palCS:SetValue(YayMounts:GetDbValue("settings", "PALADIN"))
    palCS:SetCallback("OnValueChanged",  function() YayMounts:SetDbValue("settings", "PALADIN", palCS:GetValue()) end)
    palCS:SetFullWidth(true)
    container:AddChild(palCS)
  
    local priestCS = AceGUI:Create("CheckBox")
    priestCS:SetLabel(YayMountsLocale.String.PL)
    priestCS:SetValue(YayMounts:GetDbValue("settings", "PRIEST"))
    priestCS:SetCallback("OnValueChanged",  function() YayMounts:SetDbValue("settings", "PRIEST", priestCS:GetValue()) end)
    priestCS:SetFullWidth(true)
    container:AddChild(priestCS)
	
	------Monk stuff-----
	local monkHeading = AceGUI:Create("Heading")
	monkHeading:SetText(YayMountsLocale.String.MONKHEADING)
	monkHeading:SetFullWidth(true)
	container:AddChild(monkHeading)
	
	local monkCS = AceGUI:Create("CheckBox")
    monkCS:SetLabel(YayMountsLocale.String.MR)
    monkCS:SetValue(YayMounts:GetDbValue("settings", "MONK"))
    monkCS:SetCallback("OnValueChanged",  function() YayMounts:SetDbValue("settings", "MONK", monkCS:GetValue()) end)
    monkCS:SetFullWidth(true)
    container:AddChild(monkCS)
	
	local monkZen = AceGUI:Create("CheckBox")
    monkZen:SetLabel(YayMountsLocale.String.MONKZENUSE)
    monkZen:SetValue(YayMounts:GetDbValue("settings", "MONKZENUSE"))
    monkZen:SetCallback("OnValueChanged",  function() YayMounts:SetDbValue("settings", "MONKZENUSE", monkZen:GetValue()) end)
    monkZen:SetFullWidth(true)
    container:AddChild(monkZen)
	
	------Druid stuff-----
	local druidHeading = AceGUI:Create("Heading")
	druidHeading:SetText(YayMountsLocale.String.DRUIDHEADING)
	druidHeading:SetFullWidth(true)
	container:AddChild(druidHeading)
	
	local flightFormPriority = AceGUI:Create("CheckBox")
    flightFormPriority:SetLabel(YayMountsLocale.String.FFP)
    flightFormPriority:SetValue(YayMounts:GetDbValue("settings", "FFP"))
    flightFormPriority:SetCallback("OnValueChanged",  function() YayMounts:SetDbValue("settings", "FFP", flightFormPriority:GetValue()) end)
    flightFormPriority:SetFullWidth(true)
    container:AddChild(flightFormPriority)
    
    local travelToTravel = AceGUI:Create("CheckBox")
    travelToTravel:SetLabel(YayMountsLocale.String.TTT)
    travelToTravel:SetValue(YayMounts:GetDbValue("settings", "TTT"))
    travelToTravel:SetCallback("OnValueChanged", function() YayMounts:SetDbValue("settings", "TTT", travelToTravel:GetValue()) end)
    travelToTravel:SetFullWidth(true)
    container:AddChild(travelToTravel)
    
    local mountToFlightForm = AceGUI:Create("CheckBox")
    mountToFlightForm:SetLabel(YayMountsLocale.String.MTFF)
    mountToFlightForm:SetValue(YayMounts:GetDbValue("settings", "MTFF"))
    mountToFlightForm:SetCallback("OnValueChanged", function() YayMounts:SetDbValue("settings", "MTFF", mountToFlightForm:GetValue()) end)
    mountToFlightForm:SetFullWidth(true)
    container:AddChild(mountToFlightForm)
end

local function DrawGroup4(container)--KEYBINDS
	container:SetLayout("Flow")
  
	local yayMountsBindingButton = AceGUI:Create("Keybinding")
	local yayMountsBindingButton2 = AceGUI:Create("Keybinding")
	local yayMountsBindingButton3 = AceGUI:Create("Keybinding")
	local yayMountsBindingButton4 = AceGUI:Create("Keybinding")
	
	yayMountsBindingButton:SetLabel(BINDING_NAME_YAYMOUNTBINDING)
	yayMountsBindingButton:SetKey(GetBindingKey("YAYMOUNTBINDING"))
	yayMountsBindingButton:SetCallback("OnKeyChanged", 
		function() 
			YayMounts:Keybinding(yayMountsBindingButton:GetKey(), 1) 
			YayMounts:UpdateBindingButtons(yayMountsBindingButton, yayMountsBindingButton2, yayMountsBindingButton3, yayMountsBindingButton4)
		end)
	container:AddChild(yayMountsBindingButton)

	
	yayMountsBindingButton2:SetLabel(BINDING_NAME_YAYMOUNTBINDING2)
	yayMountsBindingButton2:SetKey(GetBindingKey("YAYMOUNTBINDING2"))
	yayMountsBindingButton2:SetCallback("OnKeyChanged", 
		function() 
			YayMounts:Keybinding(yayMountsBindingButton2:GetKey(), 2); 
			YayMounts:UpdateBindingButtons(yayMountsBindingButton, yayMountsBindingButton2, yayMountsBindingButton3, yayMountsBindingButton4)
		end)
	container:AddChild(yayMountsBindingButton2)
  
	
	yayMountsBindingButton3:SetLabel(BINDING_NAME_YAYMOUNTBINDING3)
	yayMountsBindingButton3:SetKey(GetBindingKey("YAYMOUNTBINDING3"))
	yayMountsBindingButton3:SetCallback("OnKeyChanged", 
		function() 
			YayMounts:Keybinding(yayMountsBindingButton3:GetKey(), 3); 
			YayMounts:UpdateBindingButtons(yayMountsBindingButton, yayMountsBindingButton2, yayMountsBindingButton3, yayMountsBindingButton4)
		end)
	container:AddChild(yayMountsBindingButton3)
	
	
	yayMountsBindingButton4:SetLabel(BINDING_NAME_YAYMOUNTBINDING4)
	yayMountsBindingButton4:SetKey(GetBindingKey("YAYMOUNTBINDING4"))
	yayMountsBindingButton4:SetCallback("OnKeyChanged", 
		function() 
			YayMounts:Keybinding(yayMountsBindingButton4:GetKey(), 4); 
			YayMounts:UpdateBindingButtons(yayMountsBindingButton, yayMountsBindingButton2, yayMountsBindingButton3, yayMountsBindingButton4)
		end)
	container:AddChild(yayMountsBindingButton4)
end

local function DrawGroup5(container) --PROFILE OPTIONS
  container:SetLayout("List")
  local cPV, tempStr = 1, YayMounts.db:GetCurrentProfile()
  
  for k,v in pairs(YayMountsDB.tProfiles) do
    if v == tempStr then
      cPV = k
    end
  end
  
  local createProfile = AceGUI:Create("EditBox")
  createProfile:SetLabel("Create new Profile")
  createProfile:SetMaxLetters(15) 
  
  local currentProfile = AceGUI:Create("Dropdown")
  currentProfile:SetList(YayMountsDB.tProfiles)
  currentProfile:SetLabel("Current Profile:")
  currentProfile:SetValue(cPV)
  
  local copyFrom = AceGUI:Create("Dropdown")
  copyFrom:SetList(YayMountsDB.tProfiles)
  copyFrom:SetItemDisabled(cPV, true)
  copyFrom:SetLabel("Copy settings from:")

  
  local copyButton = AceGUI:Create("Button")
  copyButton:SetWidth(75)
  copyButton:SetText("Copy")
  copyButton:SetDisabled(true)
  
  local deleteSelect = AceGUI:Create("Dropdown")
  deleteSelect:SetList(YayMountsDB.tProfiles)
  deleteSelect:SetItemDisabled(cPV, true)
  deleteSelect:SetLabel("Delete profile:")
  
  local deleteButton = AceGUI:Create("Button")
  deleteButton:SetWidth(100)
  deleteButton:SetText("Delete")
  deleteButton:SetDisabled(true)
  
  --Callbacks
  createProfile:SetCallback("OnEnterPressed", 
	function()
		local newProfileName = createProfile:GetText()
	
		--Check if there already is a profile with this name
		for k,v in pairs(YayMountsDB.tProfiles) do
			if(newProfileName == v) then
				return
			end
		end
	
		local tempKey = cPV
		if newProfileName ~= "" then
			YayMounts.db:SetProfile(newProfileName) 

			-- Update the dropdown lists
			YayMounts:AlphaSortProfileDB()
			currentProfile:SetList(YayMountsDB.tProfiles)
			copyFrom:SetList(YayMountsDB.tProfiles)
			deleteSelect:SetList(YayMountsDB.tProfiles)
	  
			-- Select the new profile
			if currentProfile:GetValue() ~= newProfileName then
				for k,v in pairs(YayMountsDB.tProfiles) do
				  if v == newProfileName then
					tempKey = k
				  end
				end
				currentProfile:SetValue(tempKey)
				copyFrom:SetItemDisabled(tempKey, true)
				copyFrom:SetItemDisabled(cPV, false)
				deleteSelect:SetItemDisabled(tempKey, true)
				deleteSelect:SetItemDisabled(cPV, false)
				cPV = currentProfile:GetValue()
				createProfile:SetText("")
				YayMounts:ReSort()
			end
		end
	end)
  
	currentProfile:SetCallback("OnValueChanged", 
		function() 
			if currentProfile:GetValue() ~= cPV then 
				YayMounts.db:SetProfile(YayMountsDB.tProfiles[currentProfile:GetValue()])
				copyFrom:SetItemDisabled(currentProfile:GetValue(), true)
				copyFrom:SetItemDisabled(cPV, false)
				deleteSelect:SetItemDisabled(currentProfile:GetValue(), true)
				deleteSelect:SetItemDisabled(cPV, false)
				deleteSelect:SetValue(nil)
				cPV = currentProfile:GetValue() 
			end 
		end)
  
	copyFrom:SetCallback("OnValueChanged", 
		function() 
			copyButton:SetDisabled(false) 
		end)
  
	copyButton:SetCallback("OnClick", 
		function() 
			YayMounts.db:CopyProfile(YayMountsDB.tProfiles[copyFrom:GetValue()], true) 
			copyFrom:SetValue(nil) 
			copyButton:SetDisabled(true)
			YayMounts:ReSort()
		end)
  
  deleteSelect:SetCallback("OnValueChanged", 
	function()
		if deleteSelect:GetValue() ~= nil then
		  deleteButton:SetDisabled(false) 
		else
		  deleteButton:SetDisabled(true)
		end
	end)
  
  deleteButton:SetCallback("OnClick", 
	function()
		if(copyFrom:GetValue() == deleteSelect:GetValue()) then
			copyFrom:SetValue(nil)
		end
		
		YayMounts.db:DeleteProfile(YayMountsDB.tProfiles[deleteSelect:GetValue()], true)
		deleteSelect:SetValue(nil) 
		deleteButton:SetDisabled(true)
		YayMounts:AlphaSortProfileDB()
		currentProfile:SetList(YayMountsDB.tProfiles)
		copyFrom:SetList(YayMountsDB.tProfiles)
		deleteSelect:SetList(YayMountsDB.tProfiles)
		deleteSelect:SetItemDisabled(1, true)
		deleteSelect:SetItemDisabled(cPV, true)
	end)
  
  container:AddChild(createProfile)
  container:AddChild(currentProfile)
  container:AddChild(copyFrom)
  container:AddChild(copyButton)
  container:AddChild(deleteSelect)
  container:AddChild(deleteButton)
end

local function SelectGroup(container, event, group)-- Callback function for OnGroupSelected
  container:ReleaseChildren()
  if group == "tab1" then
    DrawGroup1(container)
  elseif group == "tab2" then
    DrawGroup2(container)
  elseif group == "tab3" then
    DrawGroup3(container)
  elseif group == "tab4" then
    DrawGroup4(container)
	elseif group == "tab5" then
    DrawGroup5(container)
  end
  currentTab = group
end

function YayMounts:Frames(selectedTab)-- Basic frame
  if not yayFrame then 
    yayFrame = AceGUI:Create("YMFrame")
	tinsert(UISpecialFrames, "YMFrame")
	
    yayFrame:SetCallback("OnClose", function(widget) AceGUI:Release(widget); yayFrame = nil end)
    yayFrame:SetTitle("Yay Mounts")
    yayFrame:SetStatusText("Version "..version..", by Cyrae - Windrunner US & Anzu - Kirin Tor US")
    yayFrame:SetLayout("Fill")
    yayFrame:SetWidth(720)
    yayFrame:SetHeight(490)
    
    local tTabs = {
		{text="Mounts ("..self.db.profile.misc.NumMounts["Total"]..")", value="tab1"}, 
		{text="Mount Options", value="tab2"}, 
		{text="Class Options", value="tab3"}, 
		{text="Keybinds", value="tab4"},
		{text="Profile Options", value="tab5"},
    }
  
    --if YayMountsState.class == "MAGE" or YayMountsState.class == "PRIEST" or YayMountsState.class == "DRUID" or YayMountsState.class == "MONK" or YayMountsState.class == "ROGUE" then
		--tinsert(tTabs, 3, {text=""..YayMountsState.localClass.." Options", value = "tab3"})
    --end
  
    local tab = AceGUI:Create("TabGroup")
    tab:SetLayout("Flow")
    tab:SetTabs(tTabs)
    tab:SetCallback("OnGroupSelected", SelectGroup)
    tab:SelectTab(selectedTab)
    yayFrame:AddChild(tab)
    
  else
    AceGUI:Release(yayFrame)
    yayFrame, currentTab, currentSubTab = nil
  end
end
